@@ -306,14 +306,17 @@ void AsyncAppender::append(const spi::LoggingEventPtr& event, Pool& p)
 			auto index = oldEventCount % priv->buffer.size();
 			// Wait for a free slot
 			while (priv->bufferSize <= oldEventCount - priv->dispatchedCount)
-				;
+				std::this_thread::yield(); // Allow the dispatch thread to free a slot
 			// Write to the ring buffer
 			priv->buffer[index] = AsyncAppenderPriv::EventData{event, pendingCount};
 			// Notify the dispatch thread that an event has been added
+			auto failureCount = 0;
 			auto savedEventCount = oldEventCount;
 			while (!priv->commitCount.compare_exchange_weak(oldEventCount, oldEventCount + 1, std::memory_order_release))
 			{
-				 oldEventCount = savedEventCount;
+				oldEventCount = savedEventCount;
+				if (2 < ++failureCount) // Did the scheduler suspend a thread between claiming a slot and advancing commitCount?
+					std::this_thread::yield(); // Wait a bit
 			}
 			priv->bufferNotEmpty.notify_all();
 			break;
