@@ -15,10 +15,6 @@
  * limitations under the License.
  */
 
-#if defined(_MSC_VER)
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
-
 #include <log4cxx/logstring.h>
 #include <log4cxx/helpers/class.h>
 #include <log4cxx/helpers/exception.h>
@@ -30,8 +26,6 @@
 	#define LOG4CXX 1
 #endif
 #include <log4cxx/private/log4cxx_private.h>
-#include <log4cxx/rollingfileappender.h>
-#include <log4cxx/dailyrollingfileappender.h>
 
 
 #include <log4cxx/asyncappender.h>
@@ -45,8 +39,6 @@
 	#include <log4cxx/nt/outputdebugstringappender.h>
 #endif
 #include <log4cxx/net/smtpappender.h>
-#include <log4cxx/net/socketappender.h>
-#include <log4cxx/net/sockethubappender.h>
 #include <log4cxx/helpers/datagramsocket.h>
 #include <log4cxx/net/syslogappender.h>
 #include <log4cxx/net/telnetappender.h>
@@ -54,14 +46,18 @@
 #include <log4cxx/net/xmlsocketappender.h>
 #include <log4cxx/layout.h>
 #include <log4cxx/patternlayout.h>
+#include <log4cxx/jsonlayout.h>
 #include <log4cxx/htmllayout.h>
 #include <log4cxx/simplelayout.h>
 #include <log4cxx/xml/xmllayout.h>
-#include <log4cxx/ttcclayout.h>
+#if LOG4CXX_HAS_FMT_LAYOUT
+#include <log4cxx/fmtlayout.h>
+#endif
 
 #include <log4cxx/filter/levelmatchfilter.h>
 #include <log4cxx/filter/levelrangefilter.h>
 #include <log4cxx/filter/stringmatchfilter.h>
+#include <log4cxx/filter/locationinfofilter.h>
 #include <log4cxx/rolling/filterbasedtriggeringpolicy.h>
 #include <log4cxx/rolling/fixedwindowrollingpolicy.h>
 #include <log4cxx/rolling/manualtriggeringpolicy.h>
@@ -72,20 +68,30 @@
 #include <log4cxx/xml/domconfigurator.h>
 #include <log4cxx/propertyconfigurator.h>
 #include <log4cxx/varia/fallbackerrorhandler.h>
-#include <apr.h>
 
 
-using namespace log4cxx;
-using namespace log4cxx::helpers;
-using namespace log4cxx::net;
-using namespace log4cxx::filter;
-using namespace log4cxx::xml;
-using namespace log4cxx::rolling;
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::helpers;
+using namespace LOG4CXX_NS::net;
+using namespace LOG4CXX_NS::filter;
+using namespace LOG4CXX_NS::xml;
+using namespace LOG4CXX_NS::rolling;
 
-log4cxx_uint32_t libraryVersion(){
+namespace LOG4CXX_NS
+{
+uint32_t libraryVersion()
+{
 	// This function defined in log4cxx.h
 	return LOG4CXX_VERSION;
 }
+}
+
+#if LOG4CXX_ABI_VERSION <= 15
+LOG4CXX_EXPORT uint32_t libraryVersion()
+{
+	return  LOG4CXX_NS::libraryVersion();
+}
+#endif
 
 Class::Class()
 {
@@ -95,10 +101,12 @@ Class::~Class()
 {
 }
 
+#if LOG4CXX_ABI_VERSION <= 15
 LogString Class::toString() const
 {
 	return getName();
 }
+#endif
 
 Object* Class::newInstance() const
 {
@@ -112,7 +120,7 @@ Object* Class::newInstance() const
 
 Class::ClassMap& Class::getRegistry()
 {
-	static ClassMap registry;
+	static WideLife<ClassMap> registry;
 	return registry;
 }
 
@@ -167,48 +175,44 @@ bool Class::registerClass(const Class& newClass)
 
 void Class::registerClasses()
 {
-#if APR_HAS_THREADS
 	AsyncAppender::registerClass();
-#endif
 	ConsoleAppender::registerClass();
 	FileAppender::registerClass();
-	log4cxx::db::ODBCAppender::registerClass();
+	LOG4CXX_NS::db::ODBCAppender::registerClass();
 #if (defined(WIN32) || defined(_WIN32))
 #if !defined(_WIN32_WCE)
-	log4cxx::nt::NTEventLogAppender::registerClass();
+	LOG4CXX_NS::nt::NTEventLogAppender::registerClass();
 #endif
-	log4cxx::nt::OutputDebugStringAppender::registerClass();
+	LOG4CXX_NS::nt::OutputDebugStringAppender::registerClass();
 #endif
-	log4cxx::RollingFileAppender::registerClass();
 	SMTPAppender::registerClass();
-	SocketAppender::registerClass();
-#if APR_HAS_THREADS
-	SocketHubAppender::registerClass();
-#endif
-	SyslogAppender::registerClass();
-#if APR_HAS_THREADS
-	TelnetAppender::registerClass();
-#endif
-	XMLSocketAppender::registerClass();
-	DateLayout::registerClass();
+	JSONLayout::registerClass();
 	HTMLLayout::registerClass();
+#if LOG4CXX_HAS_FMT_LAYOUT
+	FMTLayout::registerClass();
+#endif
 	PatternLayout::registerClass();
 	SimpleLayout::registerClass();
-	TTCCLayout::registerClass();
 	XMLLayout::registerClass();
 	LevelMatchFilter::registerClass();
 	LevelRangeFilter::registerClass();
 	StringMatchFilter::registerClass();
-	log4cxx::RollingFileAppender::registerClass();
-	log4cxx::rolling::RollingFileAppender::registerClass();
-	DailyRollingFileAppender::registerClass();
-	log4cxx::rolling::SizeBasedTriggeringPolicy::registerClass();
-	log4cxx::rolling::TimeBasedRollingPolicy::registerClass();
-	log4cxx::rolling::ManualTriggeringPolicy::registerClass();
-	log4cxx::rolling::FixedWindowRollingPolicy::registerClass();
-	log4cxx::rolling::FilterBasedTriggeringPolicy::registerClass();
-	log4cxx::xml::DOMConfigurator::registerClass();
-	log4cxx::PropertyConfigurator::registerClass();
-	log4cxx::varia::FallbackErrorHandler::registerClass();
+	LocationInfoFilter::registerClass();
+	LOG4CXX_NS::rolling::RollingFileAppender::registerClass();
+	LOG4CXX_NS::rolling::SizeBasedTriggeringPolicy::registerClass();
+	LOG4CXX_NS::rolling::TimeBasedRollingPolicy::registerClass();
+	LOG4CXX_NS::rolling::ManualTriggeringPolicy::registerClass();
+	LOG4CXX_NS::rolling::FixedWindowRollingPolicy::registerClass();
+	LOG4CXX_NS::rolling::FilterBasedTriggeringPolicy::registerClass();
+#if LOG4CXX_HAS_DOMCONFIGURATOR
+	LOG4CXX_NS::xml::DOMConfigurator::registerClass();
+#endif
+	LOG4CXX_NS::PropertyConfigurator::registerClass();
+	LOG4CXX_NS::varia::FallbackErrorHandler::registerClass();
+#if LOG4CXX_HAS_NETWORKING
+	TelnetAppender::registerClass();
+	XMLSocketAppender::registerClass();
+	SyslogAppender::registerClass();
+#endif
 }
 
