@@ -30,7 +30,11 @@
 #include <log4cxx/config/propertysetter.h>
 #include <log4cxx/spi/errorhandler.h>
 #include <log4cxx/spi/loggerfactory.h>
+#if LOG4CXX_ABI_VERSION <= 15
 #include <log4cxx/defaultloggerfactory.h>
+#else
+#include <log4cxx/spi/loggerfactory.h>
+#endif
 #include <log4cxx/helpers/filewatchdog.h>
 #include <log4cxx/spi/loggerrepository.h>
 #include <log4cxx/spi/loggingevent.h>
@@ -49,12 +53,12 @@
 #define LOG4CXX 1
 #include <log4cxx/helpers/aprinitializer.h>
 
-using namespace log4cxx;
-using namespace log4cxx::xml;
-using namespace log4cxx::helpers;
-using namespace log4cxx::spi;
-using namespace log4cxx::config;
-using namespace log4cxx::rolling;
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::xml;
+using namespace LOG4CXX_NS::helpers;
+using namespace LOG4CXX_NS::spi;
+using namespace LOG4CXX_NS::config;
+using namespace LOG4CXX_NS::rolling;
 
 #define MAX_ATTRIBUTE_NAME_LEN 2000
 
@@ -65,8 +69,7 @@ struct DOMConfigurator::DOMConfiguratorPrivate
 	spi::LoggerFactoryPtr loggerFactory;
 };
 
-#if APR_HAS_THREADS
-namespace log4cxx
+namespace LOG4CXX_NS
 {
 namespace xml
 {
@@ -90,7 +93,6 @@ class XMLWatchdog  : public FileWatchdog
 }
 }
 XMLWatchdog* DOMConfigurator::xdog = NULL;
-#endif
 
 
 IMPLEMENT_LOG4CXX_OBJECT(DOMConfigurator)
@@ -134,8 +136,8 @@ DOMConfigurator::~DOMConfigurator() {}
 /**
 Used internally to parse appenders by IDREF name.
 */
-AppenderPtr DOMConfigurator::findAppenderByName(log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+AppenderPtr DOMConfigurator::findAppenderByName(LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* element,
 	apr_xml_doc* doc,
 	const LogString& appenderName,
@@ -169,8 +171,8 @@ AppenderPtr DOMConfigurator::findAppenderByName(log4cxx::helpers::Pool& p,
  Used internally to parse appenders by IDREF element.
 */
 AppenderPtr DOMConfigurator::findAppenderByReference(
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* appenderRef,
 	apr_xml_doc* doc,
 	AppenderMap& appenders)
@@ -206,19 +208,22 @@ AppenderPtr DOMConfigurator::findAppenderByReference(
 Used internally to parse an appender element.
 */
 AppenderPtr DOMConfigurator::parseAppender(Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* appenderElement,
 	apr_xml_doc* doc,
 	AppenderMap& appenders)
 {
 
 	LogString className(subst(getAttribute(utf8Decoder, appenderElement, CLASS_ATTR)));
-	LogLog::debug(LOG4CXX_STR("Class name: [") + className + LOG4CXX_STR("]"));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Class name: [") + className + LOG4CXX_STR("]"));
+	}
 
 	try
 	{
 		ObjectPtr instance = ObjectPtr(Loader::loadClass(className).newInstance());
-		AppenderPtr appender = log4cxx::cast<Appender>(instance);
+		AppenderPtr appender = LOG4CXX_NS::cast<Appender>(instance);
 		PropertySetter propSetter(appender);
 
 		appender->setName(subst(getAttribute(utf8Decoder, appenderElement, NAME_ATTR)));
@@ -243,14 +248,12 @@ AppenderPtr DOMConfigurator::parseAppender(Pool& p,
 			// Add filters
 			else if (tagName == FILTER_TAG)
 			{
-				std::vector<log4cxx::spi::FilterPtr> filters;
+				std::vector<LOG4CXX_NS::spi::FilterPtr> filters;
 				parseFilters(p, utf8Decoder, currentElement, filters);
 
-				for (std::vector<log4cxx::spi::FilterPtr>::iterator iter = filters.begin();
-					iter != filters.end();
-					iter++)
+				for (auto& item : filters)
 				{
-					appender->addFilter(*iter);
+					appender->addFilter(item);
 				}
 			}
 			else if (tagName == ERROR_HANDLER_TAG)
@@ -260,7 +263,7 @@ AppenderPtr DOMConfigurator::parseAppender(Pool& p,
 			else if (tagName == ROLLING_POLICY_TAG)
 			{
 				RollingPolicyPtr rollPolicy(parseRollingPolicy(p, utf8Decoder, currentElement));
-				RollingFileAppenderPtr rfa = log4cxx::cast<RollingFileAppender>(appender);
+				RollingFileAppenderPtr rfa = LOG4CXX_NS::cast<RollingFileAppender>(appender);
 
 				if (rfa != NULL)
 				{
@@ -270,8 +273,8 @@ AppenderPtr DOMConfigurator::parseAppender(Pool& p,
 			else if (tagName == TRIGGERING_POLICY_TAG)
 			{
 				ObjectPtr policy(parseTriggeringPolicy(p, utf8Decoder, currentElement));
-				RollingFileAppenderPtr rfa = log4cxx::cast<RollingFileAppender>(appender);
-				TriggeringPolicyPtr policyPtr = log4cxx::cast<TriggeringPolicy>(policy);
+				RollingFileAppenderPtr rfa = LOG4CXX_NS::cast<RollingFileAppender>(appender);
+				TriggeringPolicyPtr policyPtr = LOG4CXX_NS::cast<TriggeringPolicy>(policy);
 
 				if (rfa != NULL)
 				{
@@ -279,11 +282,11 @@ AppenderPtr DOMConfigurator::parseAppender(Pool& p,
 				}
 				else
 				{
-					auto smtpa = log4cxx::cast<log4cxx::net::SMTPAppender>(appender);
+					auto smtpa = LOG4CXX_NS::cast<LOG4CXX_NS::net::SMTPAppender>(appender);
 
 					if (smtpa != NULL)
 					{
-						auto evaluator = log4cxx::cast<TriggeringEventEvaluator>(policy);
+						auto evaluator = LOG4CXX_NS::cast<TriggeringEventEvaluator>(policy);
 						smtpa->setEvaluator(evaluator);
 					}
 				}
@@ -294,10 +297,13 @@ AppenderPtr DOMConfigurator::parseAppender(Pool& p,
 
 				if (appender->instanceof(AppenderAttachable::getStaticClass()))
 				{
-					AppenderAttachablePtr aa = log4cxx::cast<AppenderAttachable>(appender);
-					LogLog::debug(LOG4CXX_STR("Attaching appender named [") +
-						refName + LOG4CXX_STR("] to appender named [") +
-						appender->getName() + LOG4CXX_STR("]."));
+					AppenderAttachablePtr aa = LOG4CXX_NS::cast<AppenderAttachable>(appender);
+					if (LogLog::isDebugEnabled())
+					{
+						LogLog::debug(LOG4CXX_STR("Attaching appender named [") +
+							refName + LOG4CXX_STR("] to appender named [") +
+							appender->getName() + LOG4CXX_STR("]."));
+					}
 					aa->addAppender(findAppenderByReference(p, utf8Decoder, currentElement, doc, appenders));
 				}
 				else
@@ -326,7 +332,7 @@ AppenderPtr DOMConfigurator::parseAppender(Pool& p,
 Used internally to parse an {@link ErrorHandler} element.
 */
 void DOMConfigurator::parseErrorHandler(Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* element,
 	AppenderPtr& appender,
 	apr_xml_doc* doc,
@@ -338,7 +344,7 @@ void DOMConfigurator::parseErrorHandler(Pool& p,
 			subst(getAttribute(utf8Decoder, element, CLASS_ATTR)),
 			ErrorHandler::getStaticClass(),
 			0);
-	eh = log4cxx::cast<ErrorHandler>(obj);
+	eh = LOG4CXX_NS::cast<ErrorHandler>(obj);
 
 	if (eh != 0)
 	{
@@ -374,7 +380,7 @@ void DOMConfigurator::parseErrorHandler(Pool& p,
 		}
 
 		propSetter.activate(p);
-		std::shared_ptr<AppenderSkeleton> appSkeleton = log4cxx::cast<AppenderSkeleton>(appender);
+		std::shared_ptr<AppenderSkeleton> appSkeleton = LOG4CXX_NS::cast<AppenderSkeleton>(appender);
 
 		if (appSkeleton != 0)
 		{
@@ -387,15 +393,15 @@ void DOMConfigurator::parseErrorHandler(Pool& p,
  Used internally to parse a filter element.
 */
 void DOMConfigurator::parseFilters(Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* element,
-	std::vector<log4cxx::spi::FilterPtr>& filters)
+	std::vector<LOG4CXX_NS::spi::FilterPtr>& filters)
 {
 	LogString clazz = subst(getAttribute(utf8Decoder, element, CLASS_ATTR));
 	FilterPtr filter;
 	std::shared_ptr<Object> obj = OptionConverter::instantiateByClassName(clazz,
 			Filter::getStaticClass(), 0);
-	filter = log4cxx::cast<Filter>(obj);
+	filter = LOG4CXX_NS::cast<Filter>(obj);
 
 	if (filter != 0)
 	{
@@ -422,8 +428,8 @@ void DOMConfigurator::parseFilters(Pool& p,
 Used internally to parse an category or logger element.
 */
 void DOMConfigurator::parseLogger(
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* loggerElement,
 	apr_xml_doc* doc,
 	AppenderMap& appenders)
@@ -431,7 +437,10 @@ void DOMConfigurator::parseLogger(
 	// Create a new Logger object from the <category> element.
 	LogString loggerName = subst(getAttribute(utf8Decoder, loggerElement, NAME_ATTR));
 
-	LogLog::debug(LOG4CXX_STR("Retreiving an instance of Logger."));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Retreiving an instance of ") + loggerName);
+	}
 	LoggerPtr logger = m_priv->repository->getLogger(loggerName, m_priv->loggerFactory);
 
 	// Setting up a logger needs to be an atomic operation, in order
@@ -441,8 +450,11 @@ void DOMConfigurator::parseLogger(
 			subst(getAttribute(utf8Decoder, loggerElement, ADDITIVITY_ATTR)),
 			true);
 
-	LogLog::debug(LOG4CXX_STR("Setting [") + logger->getName() + LOG4CXX_STR("] additivity to [") +
-		(additivity ? LogString(LOG4CXX_STR("true")) : LogString(LOG4CXX_STR("false"))) + LOG4CXX_STR("]."));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Setting [") + logger->getName() + LOG4CXX_STR("] additivity to [") +
+			(additivity ? LogString(LOG4CXX_STR("true")) : LogString(LOG4CXX_STR("false"))) + LOG4CXX_STR("]."));
+	}
 	logger->setAdditivity(additivity);
 	parseChildrenOfLoggerElement(p, utf8Decoder, loggerElement, logger, false, doc, appenders);
 }
@@ -451,8 +463,8 @@ void DOMConfigurator::parseLogger(
  Used internally to parse the logger factory element.
 */
 void DOMConfigurator::parseLoggerFactory(
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* factoryElement)
 {
 	LogString className(subst(getAttribute(utf8Decoder, factoryElement, CLASS_ATTR)));
@@ -460,16 +472,18 @@ void DOMConfigurator::parseLoggerFactory(
 	if (className.empty())
 	{
 		LogLog::error(LOG4CXX_STR("Logger Factory tag class attribute not found."));
-		LogLog::debug(LOG4CXX_STR("No Logger Factory configured."));
 	}
 	else
 	{
-		LogLog::debug(LOG4CXX_STR("Desired logger factory: [") + className + LOG4CXX_STR("]"));
+		if (LogLog::isDebugEnabled())
+		{
+			LogLog::debug(LOG4CXX_STR("Desired logger factory: [") + className + LOG4CXX_STR("]"));
+		}
 		std::shared_ptr<Object> obj = OptionConverter::instantiateByClassName(
 				className,
 				LoggerFactory::getStaticClass(),
 				0);
-		m_priv->loggerFactory = log4cxx::cast<LoggerFactory>(obj);
+		m_priv->loggerFactory = LOG4CXX_NS::cast<LoggerFactory>(obj);
 		PropertySetter propSetter(m_priv->loggerFactory);
 
 		for (apr_xml_elem* currentElement = factoryElement->first_child;
@@ -490,8 +504,8 @@ void DOMConfigurator::parseLoggerFactory(
  Used internally to parse the root logger element.
 */
 void DOMConfigurator::parseRoot(
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* rootElement,
 	apr_xml_doc* doc,
 	AppenderMap& appenders)
@@ -504,8 +518,8 @@ void DOMConfigurator::parseRoot(
  Used internally to parse the children of a logger element.
 */
 void DOMConfigurator::parseChildrenOfLoggerElement(
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* loggerElement, LoggerPtr logger, bool isRoot,
 	apr_xml_doc* doc,
 	AppenderMap& appenders)
@@ -528,7 +542,9 @@ void DOMConfigurator::parseChildrenOfLoggerElement(
 			AppenderPtr appender = findAppenderByReference(p, utf8Decoder, currentElement, doc, appenders);
 			LogString refName =  subst(getAttribute(utf8Decoder, currentElement, REF_ATTR));
 
-			if (appender != 0)
+			if (!LogLog::isDebugEnabled())
+				;
+			else if (appender != 0)
 			{
 				LogLog::debug(LOG4CXX_STR("Adding appender named [") + refName +
 					LOG4CXX_STR("] to logger [") + logger->getName() + LOG4CXX_STR("]."));
@@ -563,17 +579,20 @@ void DOMConfigurator::parseChildrenOfLoggerElement(
  Used internally to parse a layout element.
 */
 LayoutPtr DOMConfigurator::parseLayout (
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* layout_element)
 {
 	LogString className(subst(getAttribute(utf8Decoder, layout_element, CLASS_ATTR)));
-	LogLog::debug(LOG4CXX_STR("Parsing layout of class: \"") + className + LOG4CXX_STR("\""));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Parsing layout of class: \"") + className + LOG4CXX_STR("\""));
+	}
 
 	try
 	{
 		ObjectPtr instance = ObjectPtr(Loader::loadClass(className).newInstance());
-		LayoutPtr layout = log4cxx::cast<Layout>(instance);
+		LayoutPtr layout = LOG4CXX_NS::cast<Layout>(instance);
 		PropertySetter propSetter(layout);
 
 		for (apr_xml_elem* currentElement = layout_element->first_child;
@@ -603,12 +622,15 @@ LayoutPtr DOMConfigurator::parseLayout (
  Used internally to parse a triggering policy
 */
 ObjectPtr DOMConfigurator::parseTriggeringPolicy (
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* layout_element)
 {
 	LogString className = subst(getAttribute(utf8Decoder, layout_element, CLASS_ATTR));
-	LogLog::debug(LOG4CXX_STR("Parsing triggering policy of class: \"") + className + LOG4CXX_STR("\""));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Parsing triggering policy of class: \"") + className + LOG4CXX_STR("\""));
+	}
 
 	try
 	{
@@ -627,17 +649,15 @@ ObjectPtr DOMConfigurator::parseTriggeringPolicy (
 			}
 			else if (tagName == FILTER_TAG)
 			{
-				std::vector<log4cxx::spi::FilterPtr> filters;
+				std::vector<LOG4CXX_NS::spi::FilterPtr> filters;
 				parseFilters(p, utf8Decoder, currentElement, filters);
-				FilterBasedTriggeringPolicyPtr fbtp = log4cxx::cast<FilterBasedTriggeringPolicy>(instance);
+				FilterBasedTriggeringPolicyPtr fbtp = LOG4CXX_NS::cast<FilterBasedTriggeringPolicy>(instance);
 
 				if (fbtp != NULL)
 				{
-					for (std::vector<log4cxx::spi::FilterPtr>::iterator iter = filters.begin();
-						iter != filters.end();
-						iter++)
+					for (auto& item : filters)
 					{
-						fbtp->addFilter(*iter);
+						fbtp->addFilter(item);
 					}
 				}
 			}
@@ -658,17 +678,20 @@ ObjectPtr DOMConfigurator::parseTriggeringPolicy (
  Used internally to parse a triggering policy
 */
 RollingPolicyPtr DOMConfigurator::parseRollingPolicy (
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* layout_element)
 {
 	LogString className = subst(getAttribute(utf8Decoder, layout_element, CLASS_ATTR));
-	LogLog::debug(LOG4CXX_STR("Parsing rolling policy of class: \"") + className + LOG4CXX_STR("\""));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Parsing rolling policy of class: \"") + className + LOG4CXX_STR("\""));
+	}
 
 	try
 	{
 		ObjectPtr instance = ObjectPtr(Loader::loadClass(className).newInstance());
-		RollingPolicyPtr layout = log4cxx::cast<RollingPolicy>(instance);
+		RollingPolicyPtr layout = LOG4CXX_NS::cast<RollingPolicy>(instance);
 		PropertySetter propSetter(layout);
 
 		for (apr_xml_elem* currentElement = layout_element->first_child;
@@ -700,8 +723,8 @@ RollingPolicyPtr DOMConfigurator::parseRollingPolicy (
  Used internally to parse a level  element.
 */
 void DOMConfigurator::parseLevel(
-	log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* element, LoggerPtr logger, bool isRoot)
 {
 	LogString loggerName = logger->getName();
@@ -712,7 +735,10 @@ void DOMConfigurator::parseLevel(
 	}
 
 	LogString levelStr(subst(getAttribute(utf8Decoder, element, VALUE_ATTR)));
-	LogLog::debug(LOG4CXX_STR("Level value for ") + loggerName + LOG4CXX_STR(" is [") + levelStr + LOG4CXX_STR("]."));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Level value for ") + loggerName + LOG4CXX_STR(" is [") + levelStr + LOG4CXX_STR("]."));
+	}
 
 	if (StringHelper::equalsIgnoreCase(levelStr, LOG4CXX_STR("INHERITED"), LOG4CXX_STR("inherited"))
 		|| StringHelper::equalsIgnoreCase(levelStr, LOG4CXX_STR("NULL"), LOG4CXX_STR("null")))
@@ -736,7 +762,10 @@ void DOMConfigurator::parseLevel(
 		}
 		else
 		{
-			LogLog::debug(LOG4CXX_STR("Desired Level sub-class: [") + className + LOG4CXX_STR("]"));
+			if (LogLog::isDebugEnabled())
+			{
+				LogLog::debug(LOG4CXX_STR("Desired Level sub-class: [") + className + LOG4CXX_STR("]"));
+			}
 
 			try
 			{
@@ -764,12 +793,15 @@ void DOMConfigurator::parseLevel(
 		}
 	}
 
-	LogLog::debug(loggerName + LOG4CXX_STR(" level set to ") +
-		logger->getEffectiveLevel()->toString());
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(loggerName + LOG4CXX_STR(" level set to ") +
+			logger->getEffectiveLevel()->toString());
+	}
 }
 
-void DOMConfigurator::setParameter(log4cxx::helpers::Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+void DOMConfigurator::setParameter(LOG4CXX_NS::helpers::Pool& p,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* elem,
 	PropertySetter& propSetter)
 {
@@ -779,16 +811,30 @@ void DOMConfigurator::setParameter(log4cxx::helpers::Pool& p,
 	propSetter.setProperty(name, value, p);
 }
 
-spi::ConfigurationStatus DOMConfigurator::doConfigure(const File& filename, spi::LoggerRepositoryPtr repository1)
+spi::ConfigurationStatus DOMConfigurator::doConfigure
+	( const File&                     filename
+#if LOG4CXX_ABI_VERSION <= 15
+	, spi::LoggerRepositoryPtr        repository
+#else
+	, const spi::LoggerRepositoryPtr& repository
+#endif
+	)
 {
-	repository1->setConfigured(true);
-	m_priv->repository = repository1;
-	LogString msg(LOG4CXX_STR("DOMConfigurator configuring file "));
-	msg.append(filename.getPath());
-	msg.append(LOG4CXX_STR("..."));
-	LogLog::debug(msg);
+	m_priv->repository = repository ? repository : LogManager::getLoggerRepository();
+	m_priv->repository->setConfigured(true);
+	if (LogLog::isDebugEnabled())
+	{
+		LogString msg(LOG4CXX_STR("DOMConfigurator configuring file "));
+		msg.append(filename.getPath());
+		msg.append(LOG4CXX_STR("..."));
+		LogLog::debug(msg);
+	}
 
+#if LOG4CXX_ABI_VERSION <= 15
 	m_priv->loggerFactory = std::make_shared<DefaultLoggerFactory>();
+#else
+	m_priv->loggerFactory = std::make_shared<LoggerFactory>();
+#endif
 
 	Pool p;
 	apr_file_t* fd;
@@ -813,9 +859,12 @@ spi::ConfigurationStatus DOMConfigurator::doConfigure(const File& filename, spi:
 		apr_xml_parser* parser = NULL;
 		apr_xml_doc* doc = NULL;
 
-		LogString debugMsg = LOG4CXX_STR("Loading configuration file [")
-				+ filename.getPath() + LOG4CXX_STR("].");
-		LogLog::debug(debugMsg);
+		if (LogLog::isDebugEnabled())
+		{
+			LogString debugMsg = LOG4CXX_STR("Loading configuration file [")
+					+ filename.getPath() + LOG4CXX_STR("].");
+			LogLog::debug(debugMsg);
+		}
 
 		rv = apr_xml_parse_file(p.getAPRPool(), &parser, &doc, fd, 2000);
 
@@ -851,6 +900,13 @@ spi::ConfigurationStatus DOMConfigurator::doConfigure(const File& filename, spi:
 	return spi::ConfigurationStatus::Configured;
 }
 
+// Read configuration options from \c filename.
+spi::ConfigurationStatus DOMConfigurator::configure(const File& filename)
+{
+	return DOMConfigurator().doConfigure(filename, LogManager::getLoggerRepository());
+}
+
+#if LOG4CXX_ABI_VERSION <= 15
 spi::ConfigurationStatus DOMConfigurator::configure(const std::string& filename)
 {
 	File file(filename);
@@ -865,7 +921,7 @@ spi::ConfigurationStatus DOMConfigurator::configure(const std::wstring& filename
 }
 #endif
 
-#if LOG4CXX_UNICHAR_API
+#if LOG4CXX_UNICHAR_API || LOG4CXX_LOGCHAR_IS_UNICHAR
 spi::ConfigurationStatus DOMConfigurator::configure(const std::basic_string<UniChar>& filename)
 {
 	File file(filename);
@@ -894,7 +950,7 @@ spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::wstring&
 }
 #endif
 
-#if LOG4CXX_UNICHAR_API
+#if LOG4CXX_UNICHAR_API || LOG4CXX_LOGCHAR_IS_UNICHAR
 spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::basic_string<UniChar>& filename)
 {
 	return configureAndWatch(filename, FileWatchdog::DEFAULT_DELAY);
@@ -910,9 +966,12 @@ spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const CFStringRef& f
 
 spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::string& filename, long delay)
 {
-	File file(filename);
-#if APR_HAS_THREADS
+	return configureAndWatch(File(filename), delay);
+}
+#endif // LOG4CXX_ABI_VERSION <= 15
 
+spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const File& file, long delay)
+{
 	if ( xdog )
 	{
 		APRInitializer::unregisterCleanup(xdog);
@@ -923,21 +982,17 @@ spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::string& f
 
 	xdog = new XMLWatchdog(file);
 	APRInitializer::registerCleanup(xdog);
-	xdog->setDelay(delay);
+	xdog->setDelay(0 < delay ? delay : FileWatchdog::DEFAULT_DELAY);
 	xdog->start();
 
 	return status;
-#else
-	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
-#endif
 }
 
+#if LOG4CXX_ABI_VERSION <= 15
 #if LOG4CXX_WCHAR_T_API
 spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::wstring& filename, long delay)
 {
 	File file(filename);
-#if APR_HAS_THREADS
-
 	if ( xdog )
 	{
 		APRInitializer::unregisterCleanup(xdog);
@@ -952,18 +1007,13 @@ spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::wstring&
 	xdog->start();
 
 	return status;
-#else
-	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
-#endif
 }
 #endif
 
-#if LOG4CXX_UNICHAR_API
+#if LOG4CXX_UNICHAR_API || LOG4CXX_LOGCHAR_IS_UNICHAR
 spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::basic_string<UniChar>& filename, long delay)
 {
 	File file(filename);
-#if APR_HAS_THREADS
-
 	if ( xdog )
 	{
 		APRInitializer::unregisterCleanup(xdog);
@@ -978,9 +1028,6 @@ spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::basic_str
 	xdog->start();
 
 	return status;
-#else
-	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
-#endif
 }
 #endif
 
@@ -988,8 +1035,6 @@ spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const std::basic_str
 spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const CFStringRef& filename, long delay)
 {
 	File file(filename);
-#if APR_HAS_THREADS
-
 	if ( xdog )
 	{
 		APRInitializer::unregisterCleanup(xdog);
@@ -1004,15 +1049,13 @@ spi::ConfigurationStatus DOMConfigurator::configureAndWatch(const CFStringRef& f
 	xdog->start();
 
 	return status;
-#else
-	return DOMConfigurator().doConfigure(file, LogManager::getLoggerRepository());
-#endif
 }
 #endif
+#endif // LOG4CXX_ABI_VERSION <= 15
 
 void DOMConfigurator::parse(
 	Pool& p,
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* element,
 	apr_xml_doc* doc,
 	AppenderMap& appenders)
@@ -1037,16 +1080,19 @@ void DOMConfigurator::parse(
 
 	LogString debugAttrib = subst(getAttribute(utf8Decoder, element, INTERNAL_DEBUG_ATTR));
 
-	static const LogString NULL_STRING(LOG4CXX_STR("NULL"));
-	LogLog::debug(LOG4CXX_STR("debug attribute= \"") + debugAttrib + LOG4CXX_STR("\"."));
+	static const WideLife<LogString> NULL_STRING(LOG4CXX_STR("NULL"));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("debug attribute= \"") + debugAttrib + LOG4CXX_STR("\"."));
+	}
 
 	// if the log4j.dtd is not specified in the XML file, then the
 	// "debug" attribute is returned as the empty string.
-	if (!debugAttrib.empty() && debugAttrib != NULL_STRING)
+	if (!debugAttrib.empty() && debugAttrib != NULL_STRING.value())
 	{
 		LogLog::setInternalDebugging(OptionConverter::toBoolean(debugAttrib, true));
 	}
-	else
+	else if (LogLog::isDebugEnabled())
 	{
 		LogLog::debug(LOG4CXX_STR("Ignoring internalDebug attribute."));
 	}
@@ -1054,7 +1100,7 @@ void DOMConfigurator::parse(
 
 	LogString confDebug = subst(getAttribute(utf8Decoder, element, CONFIG_DEBUG_ATTR));
 
-	if (!confDebug.empty() && confDebug != NULL_STRING)
+	if (!confDebug.empty() && confDebug != NULL_STRING.value())
 	{
 		LogLog::warn(LOG4CXX_STR("The \"configDebug\" attribute is deprecated."));
 		LogLog::warn(LOG4CXX_STR("Use the \"internalDebug\" attribute instead."));
@@ -1062,16 +1108,19 @@ void DOMConfigurator::parse(
 	}
 
 	LogString thresholdStr = subst(getAttribute(utf8Decoder, element, THRESHOLD_ATTR));
-	LogLog::debug(LOG4CXX_STR("Threshold =\"") + thresholdStr + LOG4CXX_STR("\"."));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("Threshold =\"") + thresholdStr + LOG4CXX_STR("\"."));
+	}
 
-	if (!thresholdStr.empty() && thresholdStr != NULL_STRING)
+	if (!thresholdStr.empty() && thresholdStr != NULL_STRING.value())
 	{
 		m_priv->repository->setThreshold(thresholdStr);
 	}
 
 	LogString threadSignalValue = subst(getAttribute(utf8Decoder, element, THREAD_CONFIG_ATTR));
 
-	if ( !threadSignalValue.empty() && threadSignalValue != NULL_STRING )
+	if ( !threadSignalValue.empty() && threadSignalValue != NULL_STRING.value() )
 	{
 		if ( threadSignalValue == LOG4CXX_STR("NoConfiguration") )
 		{
@@ -1137,7 +1186,7 @@ LogString DOMConfigurator::subst(const LogString& value)
 
 
 LogString DOMConfigurator::getAttribute(
-	log4cxx::helpers::CharsetDecoderPtr& utf8Decoder,
+	LOG4CXX_NS::helpers::CharsetDecoderPtr& utf8Decoder,
 	apr_xml_elem* element,
 	const std::string& attrName)
 {
