@@ -36,9 +36,9 @@
 #endif
 #include <log4cxx/helpers/aprinitializer.h>
 
-using namespace log4cxx;
-using namespace log4cxx::spi;
-using namespace log4cxx::helpers;
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::spi;
+using namespace LOG4CXX_NS::helpers;
 
 IMPLEMENT_LOG4CXX_OBJECT(DefaultRepositorySelector)
 
@@ -203,7 +203,6 @@ LoggerList LogManager::getCurrentLoggers()
 void LogManager::shutdown()
 {
 	APRInitializer::unregisterAll();
-	LoggerRepositoryPtr repPtr = getLoggerRepository();
 	getLoggerRepository()->shutdown();
 }
 
@@ -211,3 +210,15 @@ void LogManager::resetConfiguration()
 {
 	getLoggerRepository()->resetConfiguration();
 }
+
+bool LogManager::removeLogger(const LogString& name, bool ifNotUsed)
+{
+#if LOG4CXX_ABI_VERSION <= 15
+	bool result = false;
+	if (auto r = dynamic_cast<Hierarchy*>(getLoggerRepository().get()))
+		result = r->removeLogger(name, ifNotUsed);
+	return result;
+#else
+	return getLoggerRepository()->removeLogger(name, ifNotUsed);
+#endif
+}
