@@ -17,20 +17,20 @@
 
 #include <log4cxx/logstring.h>
 #include <log4cxx/propertyconfigurator.h>
-#include <log4cxx/spi/loggerfactory.h>
 #include <log4cxx/helpers/properties.h>
 #include <log4cxx/helpers/loglog.h>
 #include <log4cxx/helpers/exception.h>
 #include <log4cxx/logmanager.h>
 #include <log4cxx/helpers/optionconverter.h>
 #include <log4cxx/level.h>
+#if LOG4CXX_ABI_VERSION <= 15
 #include <log4cxx/defaultloggerfactory.h>
+#else
+#include <log4cxx/spi/loggerfactory.h>
+#endif
 #include <log4cxx/helpers/stringhelper.h>
-#include <log4cxx/appender.h>
-#include <log4cxx/logger.h>
 #include <log4cxx/layout.h>
 #include <log4cxx/config/propertysetter.h>
-#include <log4cxx/spi/loggerrepository.h>
 #include <log4cxx/helpers/stringtokenizer.h>
 #include <log4cxx/helpers/transcoder.h>
 #include <log4cxx/helpers/fileinputstream.h>
@@ -40,18 +40,15 @@
 
 #define LOG4CXX 1
 #include <log4cxx/helpers/aprinitializer.h>
-#include <apr.h>
 
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::spi;
+using namespace LOG4CXX_NS::helpers;
+using namespace LOG4CXX_NS::config;
+using namespace LOG4CXX_NS::rolling;
 
-using namespace log4cxx;
-using namespace log4cxx::spi;
-using namespace log4cxx::helpers;
-using namespace log4cxx::config;
-using namespace log4cxx::rolling;
-
-#if APR_HAS_THREADS
 #include <log4cxx/helpers/filewatchdog.h>
-namespace log4cxx
+namespace LOG4CXX_NS
 {
 class PropertyWatchdog  : public FileWatchdog
 {
@@ -75,12 +72,15 @@ class PropertyWatchdog  : public FileWatchdog
 
 PropertyWatchdog* PropertyConfigurator::pdog = NULL;
 
-#endif
-
 IMPLEMENT_LOG4CXX_OBJECT(PropertyConfigurator)
 
 PropertyConfigurator::PropertyConfigurator()
-	: registry(new std::map<LogString, AppenderPtr>()), loggerFactory(new DefaultLoggerFactory())
+	: registry(new std::map<LogString, AppenderPtr>())
+#if LOG4CXX_ABI_VERSION <= 15
+	, loggerFactory(new DefaultLoggerFactory())
+#else
+	, loggerFactory(new LoggerFactory())
+#endif
 {
 }
 
@@ -89,9 +89,16 @@ PropertyConfigurator::~PropertyConfigurator()
 	delete registry;
 }
 
-spi::ConfigurationStatus PropertyConfigurator::doConfigure(const File& configFileName,
-	spi::LoggerRepositoryPtr hierarchy)
+spi::ConfigurationStatus PropertyConfigurator::doConfigure
+	( const File&                     configFileName
+#if LOG4CXX_ABI_VERSION <= 15
+	, spi::LoggerRepositoryPtr        repository
+#else
+	, const spi::LoggerRepositoryPtr& repository
+#endif
+	)
 {
+	auto hierarchy = repository ? repository : LogManager::getLoggerRepository();
 	hierarchy->setConfigured(true);
 
 	Properties props;
@@ -103,22 +110,26 @@ spi::ConfigurationStatus PropertyConfigurator::doConfigure(const File& configFil
 	}
 	catch (const IOException& ex)
 	{
+		LOG4CXX_DECODE_CHAR(lsMsg, ex.what());
 		LogLog::error(((LogString) LOG4CXX_STR("Could not read configuration file ["))
-			+ configFileName.getPath() + LOG4CXX_STR("].") + ": " + ex.what());
+			+ configFileName.getPath() + LOG4CXX_STR("]: ") + lsMsg);
 		return spi::ConfigurationStatus::NotConfigured;
 	}
 
 	try
 	{
-		LogString debugMsg = LOG4CXX_STR("Loading configuration file [")
-				+ configFileName.getPath() + LOG4CXX_STR("].");
-		LogLog::debug(debugMsg);
+		if (LogLog::isDebugEnabled())
+		{
+			LogString debugMsg = LOG4CXX_STR("Loading configuration file [")
+					+ configFileName.getPath() + LOG4CXX_STR("].");
+			LogLog::debug(debugMsg);
+		}
 		return doConfigure(props, hierarchy);
 	}
 	catch (const std::exception& ex)
 	{
 		LogLog::error(((LogString) LOG4CXX_STR("Could not parse configuration file ["))
-			+ configFileName.getPath() + LOG4CXX_STR("]."), ex);
+			+ configFileName.getPath() + LOG4CXX_STR("]: "), ex);
 	}
 
 	return spi::ConfigurationStatus::NotConfigured;
@@ -134,13 +145,12 @@ spi::ConfigurationStatus PropertyConfigurator::configure(helpers::Properties& pr
 	return PropertyConfigurator().doConfigure(properties, LogManager::getLoggerRepository());
 }
 
-#if APR_HAS_THREADS
+#if LOG4CXX_ABI_VERSION <= 15
 spi::ConfigurationStatus PropertyConfigurator::configureAndWatch(const File& configFilename)
 {
 	return configureAndWatch(configFilename, FileWatchdog::DEFAULT_DELAY);
 }
-
-
+#endif
 
 spi::ConfigurationStatus PropertyConfigurator::configureAndWatch(
 	const File& configFilename, long delay)
@@ -155,19 +165,18 @@ spi::ConfigurationStatus PropertyConfigurator::configureAndWatch(
 
 	pdog = new PropertyWatchdog(configFilename);
 	APRInitializer::registerCleanup(pdog);
-	pdog->setDelay(delay);
+	pdog->setDelay(0 < delay ? delay : FileWatchdog::DEFAULT_DELAY);
 	pdog->start();
 
 	return stat;
 }
-#endif
 
 spi::ConfigurationStatus PropertyConfigurator::doConfigure(helpers::Properties& properties,
 	spi::LoggerRepositoryPtr hierarchy)
 {
 	hierarchy->setConfigured(true);
 
-	static const LogString DEBUG_KEY(LOG4CXX_STR("log4j.debug"));
+	static const WideLife<LogString> DEBUG_KEY(LOG4CXX_STR("log4j.debug"));
 	LogString value(properties.getProperty(DEBUG_KEY));
 
 	if (!value.empty())
@@ -175,16 +184,19 @@ spi::ConfigurationStatus PropertyConfigurator::doConfigure(helpers::Properties&
 		LogLog::setInternalDebugging(OptionConverter::toBoolean(value, true));
 	}
 
-	static const LogString THRESHOLD_PREFIX(LOG4CXX_STR("log4j.threshold"));
+	static const WideLife<LogString> THRESHOLD_PREFIX(LOG4CXX_STR("log4j.threshold"));
 	LogString thresholdStr =
 		OptionConverter::findAndSubst(THRESHOLD_PREFIX, properties);
 
 	if (!thresholdStr.empty())
 	{
 		hierarchy->setThreshold(OptionConverter::toLevel(thresholdStr, Level::getAll()));
-		LogLog::debug(((LogString) LOG4CXX_STR("Hierarchy threshold set to ["))
-			+ hierarchy->getThreshold()->toString()
-			+ LOG4CXX_STR("]."));
+		if (LogLog::isDebugEnabled())
+		{
+			LogLog::debug(((LogString) LOG4CXX_STR("Hierarchy threshold set to ["))
+				+ hierarchy->getThreshold()->toString()
+				+ LOG4CXX_STR("]."));
+		}
 	}
 
 	LogString threadConfigurationValue(properties.getProperty(LOG4CXX_STR("log4j.threadConfiguration")));
@@ -221,22 +233,25 @@ spi::ConfigurationStatus PropertyConfigurator::doConfigure(helpers::Properties&
 
 void PropertyConfigurator::configureLoggerFactory(helpers::Properties& props)
 {
-	static const LogString LOGGER_FACTORY_KEY(LOG4CXX_STR("log4j.loggerFactory"));
+	static const WideLife<LogString> LOGGER_FACTORY_KEY(LOG4CXX_STR("log4j.loggerFactory"));
 
 	LogString factoryClassName =
 		OptionConverter::findAndSubst(LOGGER_FACTORY_KEY, props);
 
 	if (!factoryClassName.empty())
 	{
-		LogString msg(LOG4CXX_STR("Setting logger factory to ["));
-		msg += factoryClassName;
-		msg += LOG4CXX_STR("].");
-		LogLog::debug(msg);
+		if (LogLog::isDebugEnabled())
+		{
+			LogString msg(LOG4CXX_STR("Setting logger factory to ["));
+			msg += factoryClassName;
+			msg += LOG4CXX_STR("].");
+			LogLog::debug(msg);
+		}
 		std::shared_ptr<Object> instance = std::shared_ptr<Object>(
 				Loader::loadClass(factoryClassName).newInstance() );
 
-		loggerFactory = log4cxx::cast<LoggerFactory>( instance );
-		static const LogString FACTORY_PREFIX(LOG4CXX_STR("log4j.factory."));
+		loggerFactory = LOG4CXX_NS::cast<LoggerFactory>( instance );
+		static const WideLife<LogString> FACTORY_PREFIX(LOG4CXX_STR("log4j.factory."));
 		Pool p;
 		PropertySetter::setProperties(loggerFactory, props, FACTORY_PREFIX, p);
 	}
@@ -245,8 +260,8 @@ void PropertyConfigurator::configureLoggerFactory(helpers::Properties& props)
 void PropertyConfigurator::configureRootLogger(helpers::Properties& props,
 	spi::LoggerRepositoryPtr& hierarchy)
 {
-	static const LogString ROOT_CATEGORY_PREFIX(LOG4CXX_STR("log4j.rootCategory"));
-	static const LogString ROOT_LOGGER_PREFIX(LOG4CXX_STR("log4j.rootLogger"));
+	static const WideLife<LogString> ROOT_CATEGORY_PREFIX(LOG4CXX_STR("log4j.rootCategory"));
+	static const WideLife<LogString> ROOT_LOGGER_PREFIX(LOG4CXX_STR("log4j.rootLogger"));
 
 
 
@@ -267,7 +282,7 @@ void PropertyConfigurator::configureRootLogger(helpers::Properties& props,
 	{
 		LoggerPtr root = hierarchy->getRootLogger();
 
-		static const LogString INTERNAL_ROOT_NAME(LOG4CXX_STR("root"));
+		static const WideLife<LogString> INTERNAL_ROOT_NAME(LOG4CXX_STR("root"));
 		parseLogger(props, root, effectiveFrefix, INTERNAL_ROOT_NAME, value, true);
 	}
 }
@@ -275,29 +290,22 @@ void PropertyConfigurator::configureRootLogger(helpers::Properties& props,
 void PropertyConfigurator::parseCatsAndRenderers(helpers::Properties& props,
 	spi::LoggerRepositoryPtr& hierarchy)
 {
-	static const LogString CATEGORY_PREFIX(LOG4CXX_STR("log4j.category."));
-	static const LogString LOGGER_PREFIX(LOG4CXX_STR("log4j.logger."));
+	static const WideLife<LogString> CATEGORY_PREFIX(LOG4CXX_STR("log4j.category."));
+	static const WideLife<LogString> LOGGER_PREFIX(LOG4CXX_STR("log4j.logger."));
 
-	std::vector<LogString> names = props.propertyNames();
-
-	std::vector<LogString>::iterator it = names.begin();
-	std::vector<LogString>::iterator itEnd = names.end();
-
-	while (it != itEnd)
+	for (auto key : props.propertyNames())
 	{
-		LogString key = *it++;
-
 		if (key.find(CATEGORY_PREFIX) == 0 || key.find(LOGGER_PREFIX) == 0)
 		{
 			LogString loggerName;
 
 			if (key.find(CATEGORY_PREFIX) == 0)
 			{
-				loggerName = key.substr(CATEGORY_PREFIX.length());
+				loggerName = key.substr(CATEGORY_PREFIX.value().length());
 			}
-			else if (key.find(LOGGER_PREFIX) == 0)
+			else if (key.find(LOGGER_PREFIX.value()) == 0)
 			{
-				loggerName = key.substr(LOGGER_PREFIX.length());
+				loggerName = key.substr(LOGGER_PREFIX.value().length());
 			}
 
 			LogString value = OptionConverter::findAndSubst(key, props);
@@ -314,22 +322,27 @@ bool PropertyConfigurator::parseAdditivityForLogger(helpers::Properties& props,
 	LoggerPtr& cat, const LogString& loggerName)
 {
 
-	static const LogString ADDITIVITY_PREFIX(LOG4CXX_STR("log4j.additivity."));
+	static const WideLife<LogString> ADDITIVITY_PREFIX(LOG4CXX_STR("log4j.additivity."));
 
 
 
-	LogString value(OptionConverter::findAndSubst(ADDITIVITY_PREFIX + loggerName, props));
-	LogLog::debug((LogString) LOG4CXX_STR("Handling ") + ADDITIVITY_PREFIX
-		+ loggerName + LOG4CXX_STR("=[") +  value + LOG4CXX_STR("]"));
-
+	LogString value(OptionConverter::findAndSubst(ADDITIVITY_PREFIX.value() + loggerName, props));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug((LogString) LOG4CXX_STR("Handling ") + ADDITIVITY_PREFIX.value()
+			+ loggerName + LOG4CXX_STR("=[") +  value + LOG4CXX_STR("]"));
+	}
 	// touch additivity only if necessary
 	if (!value.empty())
 	{
 		bool additivity = OptionConverter::toBoolean(value, true);
-		LogLog::debug(((LogString) LOG4CXX_STR("Setting additivity for \""))
-			+ loggerName
-			+ ((additivity) ?  LOG4CXX_STR("\" to true") :
-				LOG4CXX_STR("\" to false")));
+		if (LogLog::isDebugEnabled())
+		{
+			LogLog::debug(((LogString) LOG4CXX_STR("Setting additivity for \""))
+				+ loggerName
+				+ ((additivity) ?  LOG4CXX_STR("\" to true") :
+					LOG4CXX_STR("\" to false")));
+		}
 
 		return additivity;
 	}
@@ -344,10 +357,13 @@ void PropertyConfigurator::parseLogger(
 	helpers::Properties& props, LoggerPtr& logger, const LogString& /* optionKey */,
 	const LogString& loggerName, const LogString& value, bool additivity)
 {
-	LogLog::debug(((LogString) LOG4CXX_STR("Parsing for ["))
-		+ loggerName
-		+ LOG4CXX_STR("] with value=[")
-		+ value + LOG4CXX_STR("]."));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(((LogString) LOG4CXX_STR("Parsing for ["))
+			+ loggerName
+			+ LOG4CXX_STR("] with value=[")
+			+ value + LOG4CXX_STR("]."));
+	}
 
 	// We must skip over ',' but not white space
 	StringTokenizer st(value, LOG4CXX_STR(","));
@@ -363,8 +379,11 @@ void PropertyConfigurator::parseLogger(
 		}
 
 		LogString levelStr = st.nextToken();
-		LogLog::debug((LogString) LOG4CXX_STR("Level token is [")
-			+ levelStr +  LOG4CXX_STR("]."));
+		if (LogLog::isDebugEnabled())
+		{
+			LogLog::debug((LogString) LOG4CXX_STR("Level token is [")
+				+ levelStr +  LOG4CXX_STR("]."));
+		}
 
 
 		// If the level value is inherited, set logger level value to
@@ -373,26 +392,32 @@ void PropertyConfigurator::parseLogger(
 		if (StringHelper::equalsIgnoreCase(levelStr, LOG4CXX_STR("INHERITED"), LOG4CXX_STR("inherited"))
 			|| StringHelper::equalsIgnoreCase(levelStr, LOG4CXX_STR("NULL"), LOG4CXX_STR("null")))
 		{
-			static const LogString INTERNAL_ROOT_NAME(LOG4CXX_STR("root"));
+			static const WideLife<LogString> INTERNAL_ROOT_NAME(LOG4CXX_STR("root"));
 
-			if (loggerName == INTERNAL_ROOT_NAME)
+			if (loggerName == INTERNAL_ROOT_NAME.value())
 			{
 				LogLog::warn(LOG4CXX_STR("The root logger cannot be set to null."));
 			}
 			else
 			{
 				logger->setLevel(0);
-				LogLog::debug((LogString) LOG4CXX_STR("Logger ")
-					+ loggerName + LOG4CXX_STR(" set to null"));
+				if (LogLog::isDebugEnabled())
+				{
+					LogLog::debug((LogString) LOG4CXX_STR("Logger ")
+						+ loggerName + LOG4CXX_STR(" set to null"));
+				}
 			}
 		}
 		else
 		{
 			logger->setLevel(OptionConverter::toLevel(levelStr, Level::getDebug()));
 
-			LogLog::debug((LogString) LOG4CXX_STR("Logger ")
-				+ loggerName + LOG4CXX_STR(" set to ")
-				+ logger->getLevel()->toString());
+			if (LogLog::isDebugEnabled())
+			{
+				LogLog::debug((LogString) LOG4CXX_STR("Logger ")
+					+ loggerName + LOG4CXX_STR(" set to ")
+					+ logger->getLevel()->toString());
+			}
 		}
 
 	}
@@ -410,8 +435,11 @@ void PropertyConfigurator::parseLogger(
 			continue;
 		}
 
-		LogLog::debug(LOG4CXX_STR("Parsing appender named ")
-			+ appenderName + LOG4CXX_STR("\"."));
+		if (LogLog::isDebugEnabled())
+		{
+			LogLog::debug(LOG4CXX_STR("Parsing appender named ")
+				+ appenderName + LOG4CXX_STR("\"."));
+		}
 		appender = parseAppender(props, appenderName);
 
 		if (appender != 0)
@@ -430,22 +458,25 @@ AppenderPtr PropertyConfigurator::parseAppender(
 
 	if (appender != 0)
 	{
-		LogLog::debug((LogString) LOG4CXX_STR("Appender \"")
-			+ appenderName + LOG4CXX_STR("\" was already parsed."));
+		if (LogLog::isDebugEnabled())
+		{
+			LogLog::debug((LogString) LOG4CXX_STR("Appender \"")
+				+ appenderName + LOG4CXX_STR("\" was already parsed."));
+		}
 
 		return appender;
 	}
 
-	static const LogString APPENDER_PREFIX(LOG4CXX_STR("log4j.appender."));
+	static const WideLife<LogString> APPENDER_PREFIX(LOG4CXX_STR("log4j.appender."));
 
 	// Appender was not previously initialized.
-	LogString prefix = APPENDER_PREFIX + appenderName;
+	LogString prefix = APPENDER_PREFIX.value() + appenderName;
 	LogString layoutPrefix = prefix + LOG4CXX_STR(".layout");
 
 	std::shared_ptr<Object> obj =
 		OptionConverter::instantiateByKey(
 			props, prefix, Appender::getStaticClass(), 0);
-	appender = log4cxx::cast<Appender>( obj );
+	appender = LOG4CXX_NS::cast<Appender>( obj );
 
 	// Map obsolete DailyRollingFileAppender property configuration
 	if (!appender &&
@@ -476,21 +507,27 @@ AppenderPtr PropertyConfigurator::parseAppender(
 			std::shared_ptr<Object> obj =
 				OptionConverter::instantiateByKey(
 					props, layoutPrefix, Layout::getStaticClass(), 0);
-			layout = log4cxx::cast<Layout>( obj );
+			layout = LOG4CXX_NS::cast<Layout>( obj );
 
 			if (layout != 0)
 			{
 				appender->setLayout(layout);
-				LogLog::debug((LogString) LOG4CXX_STR("Parsing layout options for \"")
-					+ appenderName + LOG4CXX_STR("\"."));
+				if (LogLog::isDebugEnabled())
+				{
+					LogLog::debug((LogString) LOG4CXX_STR("Parsing layout options for \"")
+						+ appenderName + LOG4CXX_STR("\"."));
+				}
 
 				PropertySetter::setProperties(layout, props, layoutPrefix + LOG4CXX_STR("."), p);
-				LogLog::debug((LogString) LOG4CXX_STR("End of parsing for \"")
-					+ appenderName +  LOG4CXX_STR("\"."));
+				if (LogLog::isDebugEnabled())
+				{
+					LogLog::debug((LogString) LOG4CXX_STR("End of parsing for \"")
+						+ appenderName +  LOG4CXX_STR("\"."));
+				}
 			}
 		}
 
-		RollingFileAppenderPtr rolling = log4cxx::cast<rolling::RollingFileAppender>(appender);
+		RollingFileAppenderPtr rolling = LOG4CXX_NS::cast<rolling::RollingFileAppender>(appender);
 		if (rolling)
 		{
 			LogString rollingPolicyKey = prefix + LOG4CXX_STR(".rollingPolicy");
@@ -500,13 +537,16 @@ AppenderPtr PropertyConfigurator::parseAppender(
 				std::shared_ptr<Object> rolling_obj =
 					OptionConverter::instantiateByKey(
 						props, rollingPolicyKey, RollingPolicy::getStaticClass(), 0);
-				rollingPolicy = log4cxx::cast<RollingPolicy>( rolling_obj );
+				rollingPolicy = LOG4CXX_NS::cast<RollingPolicy>( rolling_obj );
 				if(rollingPolicy)
 				{
 					rolling->setRollingPolicy(rollingPolicy);
 
-					LogLog::debug((LogString) LOG4CXX_STR("Parsing rolling policy options for \"")
-						+ appenderName + LOG4CXX_STR("\"."));
+					if (LogLog::isDebugEnabled())
+					{
+						LogLog::debug((LogString) LOG4CXX_STR("Parsing rolling policy options for \"")
+							+ appenderName + LOG4CXX_STR("\"."));
+					}
 					PropertySetter::setProperties(rollingPolicy, props, rollingPolicyKey + LOG4CXX_STR("."), p);
 				}
 			}
@@ -518,21 +558,27 @@ AppenderPtr PropertyConfigurator::parseAppender(
 				std::shared_ptr<Object> triggering_obj =
 					OptionConverter::instantiateByKey(
 						props, triggeringPolicyKey, TriggeringPolicy::getStaticClass(), 0);
-				triggeringPolicy = log4cxx::cast<TriggeringPolicy>( triggering_obj );
+				triggeringPolicy = LOG4CXX_NS::cast<TriggeringPolicy>( triggering_obj );
 				if(triggeringPolicy)
 				{
 					rolling->setTriggeringPolicy(triggeringPolicy);
 
-					LogLog::debug((LogString) LOG4CXX_STR("Parsing triggering policy options for \"")
-						+ appenderName + LOG4CXX_STR("\"."));
+					if (LogLog::isDebugEnabled())
+					{
+						LogLog::debug((LogString) LOG4CXX_STR("Parsing triggering policy options for \"")
+							+ appenderName + LOG4CXX_STR("\"."));
+					}
 					PropertySetter::setProperties(triggeringPolicy, props, triggeringPolicyKey + LOG4CXX_STR("."), p);
 				}
 			}
 		}
 
 		PropertySetter::setProperties(appender, props, prefix + LOG4CXX_STR("."), p);
-		LogLog::debug((LogString) LOG4CXX_STR("Parsed \"")
-			+ appenderName + LOG4CXX_STR("\" options."));
+		if (LogLog::isDebugEnabled())
+		{
+			LogLog::debug((LogString) LOG4CXX_STR("Parsed \"")
+				+ appenderName + LOG4CXX_STR("\" options."));
+		}
 	}
 
 	registryPut(appender);
