@@ -29,9 +29,9 @@
 #include <functional>
 #include <chrono>
 
-using namespace log4cxx;
-using namespace log4cxx::helpers;
-using namespace log4cxx::net;
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::helpers;
+using namespace LOG4CXX_NS::net;
 
 #define _priv static_cast<SocketAppenderSkeletonPriv*>(m_priv.get())
 
@@ -68,25 +68,8 @@ void SocketAppenderSkeleton::activateOptions(Pool& p)
 
 void SocketAppenderSkeleton::close()
 {
-	std::lock_guard<std::recursive_mutex> lock(_priv->mutex);
-
-	if (_priv->closed)
-	{
-		return;
-	}
-
-	_priv->closed = true;
-	cleanUp(_priv->pool);
-
-	{
-		std::unique_lock<std::mutex> lock2(_priv->interrupt_mutex);
-		_priv->interrupt.notify_all();
-	}
-
-	if ( _priv->thread.joinable() )
-	{
-		_priv->thread.join();
-	}
+    _priv->stopMonitor();
+    cleanUp(_priv->pool);
 }
 
 void SocketAppenderSkeleton::connect(Pool& p)
@@ -102,18 +85,23 @@ void SocketAppenderSkeleton::connect(Pool& p)
 
 		try
 		{
+			if (LogLog::isDebugEnabled())
+			{
+				LogString msg(LOG4CXX_STR("Connecting to [")
+					+ _priv->address->toString() + LOG4CXX_STR(":"));
+				StringHelper::toString(_priv->port, p, msg);
+				msg += LOG4CXX_STR("].");
+				LogLog::debug(msg);
+			}
 			SocketPtr socket = Socket::create(_priv->address, _priv->port);
 			setSocket(socket, p);
 		}
 		catch (SocketException& e)
 		{
-			LogString msg = LOG4CXX_STR("Could not connect to remote log4cxx server at [")
-				+ _priv->address->getHostName() + LOG4CXX_STR("].");
-
-			if (_priv->reconnectionDelay > 0)
-			{
-				msg += LOG4CXX_STR(" We will try again later. ");
-			}
+			LogString msg = LOG4CXX_STR("Could not connect to [")
+				+ _priv->address->toString() + LOG4CXX_STR(":");
+			StringHelper::toString(_priv->port, p, msg);
+			msg += LOG4CXX_STR("].");
 
 			fireConnector(); // fire the connector thread
 			LogLog::error(msg, e);
@@ -159,49 +147,76 @@ void SocketAppenderSkeleton::fireConnector()
 
 void SocketAppenderSkeleton::monitor()
 {
+	Pool p;
 	SocketPtr socket;
-	bool isClosed = _priv->closed;
 
-	while (!isClosed)
+	while (!is_closed())
 	{
 		try
 		{
-			if (!_priv->closed)
+			if (LogLog::isDebugEnabled())
 			{
-				LogLog::debug(LogString(LOG4CXX_STR("Attempting connection to "))
-					+ _priv->address->getHostName());
-				socket = Socket::create(_priv->address, _priv->port);
-				Pool p;
-				setSocket(socket, p);
-				LogLog::debug(LOG4CXX_STR("Connection established. Exiting connector thread."));
-				return;
+				LogString msg(LOG4CXX_STR("Attempting connection to [")
+					+ _priv->address->toString() + LOG4CXX_STR(":"));
+				StringHelper::toString(_priv->port, p, msg);
+				msg += LOG4CXX_STR("].");
+				LogLog::debug(msg);
 			}
+			socket = Socket::create(_priv->address, _priv->port);
+			setSocket(socket, p);
+			if (LogLog::isDebugEnabled())
+			{
+				LogLog::debug(LOG4CXX_STR("Connection established. Exiting connector thread."));
+			}
+			return;
 		}
-		catch (ConnectException&)
+		catch (ConnectException& e)
 		{
-			LogLog::debug(LOG4CXX_STR("Remote host ")
-				+ _priv->address->getHostName()
-				+ LOG4CXX_STR(" refused connection."));
+			LogLog::error(LOG4CXX_STR("Remote host ")
+				+ _priv->address->toString()
+				+ LOG4CXX_STR(" refused connection."), e);
 		}
 		catch (IOException& e)
 		{
-			LogString exmsg;
-			log4cxx::helpers::Transcoder::decode(e.what(), exmsg);
-
-			LogLog::debug(((LogString) LOG4CXX_STR("Could not connect to "))
-				+ _priv->address->getHostName()
-				+ LOG4CXX_STR(". Exception is ")
-				+ exmsg);
+			LogString msg(LOG4CXX_STR("Could not connect to [")
+				+ _priv->address->toString() + LOG4CXX_STR(":"));
+			StringHelper::toString(_priv->port, p, msg);
+			msg += LOG4CXX_STR("].");
+			LogLog::error(msg, e);
 		}
 
-		std::unique_lock<std::mutex> lock( _priv->interrupt_mutex );
-		_priv->interrupt.wait_for( lock, std::chrono::milliseconds( _priv->reconnectionDelay ),
-			std::bind(&SocketAppenderSkeleton::is_closed, this) );
+		if (_priv->reconnectionDelay > 0)
+		{
+			if (LogLog::isDebugEnabled())
+			{
+				LogString msg(LOG4CXX_STR("Waiting "));
+				StringHelper::toString(_priv->reconnectionDelay, p, msg);
+				msg += LOG4CXX_STR(" ms before retrying [")
+					+ _priv->address->toString() + LOG4CXX_STR(":");
+				StringHelper::toString(_priv->port, p, msg);
+				msg += LOG4CXX_STR("].");
+				LogLog::debug(msg);
+			}
 
-		isClosed = _priv->closed;
+			std::unique_lock<std::mutex> lock( _priv->interrupt_mutex );
+			if (_priv->interrupt.wait_for( lock, std::chrono::milliseconds( _priv->reconnectionDelay ),
+				std::bind(&SocketAppenderSkeleton::is_closed, this) ))
+				break;
+		}
 	}
+}
 
-	LogLog::debug(LOG4CXX_STR("Exiting Connector.run() method."));
+void SocketAppenderSkeleton::SocketAppenderSkeletonPriv::stopMonitor()
+{
+	{
+		std::lock_guard<std::mutex> lock(this->interrupt_mutex);
+		if (this->closed)
+			return;
+		this->closed = true;
+	}
+	this->interrupt.notify_all();
+	if (this->thread.joinable())
+		this->thread.join();
 }
 
 bool SocketAppenderSkeleton::is_closed()
