@@ -23,6 +23,8 @@
 #include <log4cxx/helpers/bytebuffer.h>
 #include <log4cxx/helpers/charsetdecoder.h>
 #include <log4cxx/helpers/charsetencoder.h>
+#include <log4cxx/helpers/stringhelper.h>
+#include <log4cxx/helpers/loglog.h>
 #include <vector>
 #include <cstring>
 #if !defined(LOG4CXX)
@@ -30,12 +32,12 @@
 #endif
 #include <log4cxx/private/log4cxx_private.h>
 
-#if LOG4CXX_LOGCHAR_IS_UNICHAR || LOG4CXX_CFSTRING_API || LOG4CXX_UNICHAR_API
+#if LOG4CXX_CFSTRING_API
 	#include <CoreFoundation/CFString.h>
 #endif
 
-using namespace log4cxx;
-using namespace log4cxx::helpers;
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::helpers;
 
 
 void Transcoder::decodeUTF8(const std::string& src, LogString& dst)
@@ -574,7 +576,7 @@ void Transcoder::encode(unsigned int sv, std::wstring& dst)
 
 
 
-#if LOG4CXX_UNICHAR_API || LOG4CXX_CFSTRING_API
+#if LOG4CXX_UNICHAR_API || LOG4CXX_LOGCHAR_IS_UNICHAR
 void Transcoder::decode(const std::basic_string<UniChar>& src, LogString& dst)
 {
 #if LOG4CXX_LOGCHAR_IS_UNICHAR
@@ -623,35 +625,38 @@ void Transcoder::encode(unsigned int sv, std::basic_string<UniChar>& dst)
 #if LOG4CXX_CFSTRING_API
 void Transcoder::decode(const CFStringRef& src, LogString& dst)
 {
-	const UniChar* chars = CFStringGetCharactersPtr(src);
-
-	if (chars)
+	auto length = CFStringGetLength(src);
+#if defined(_DEBUG)
+	if (LogLog::isDebugEnabled())
 	{
-		decode(chars, dst);
+		Pool pool;
+		LogString msg(LOG4CXX_STR("Transcoder::decodeCFString"));
+		msg += LOG4CXX_STR(" length ");
+		StringHelper::toString((size_t)length, pool, msg);
+		LogLog::debug(msg);
 	}
-	else
-	{
-		size_t length = CFStringGetLength(src);
-
-		if (length > 0)
-		{
-			std::vector<UniChar> tmp(length);
-			CFStringGetCharacters(src, CFRangeMake(0, length), &tmp[0]);
-#if LOG4CXX_LOGCHAR_IS_UNICHAR
-			dst.append(&tmp[0], tmp.size());
-#else
-			decode(std::basic_string<UniChar>(&tmp[0], tmp.size()), dst);
 #endif
+
+	if (length > 0)
+	{
+		std::vector<unsigned short> tmp(length);
+		CFStringGetCharacters(src, CFRangeMake(0, length), &tmp[0]);
+		for (auto i = tmp.begin(); i != tmp.end(); )
+		{
+			unsigned int cp = decodeUTF16(tmp, i);
+			encode(cp, dst);
 		}
 	}
 }
 
 CFStringRef Transcoder::encode(const LogString& src)
 {
-	LOG4CXX_ENCODE_UNICHAR(tmp, src);
+	std::basic_string<unsigned short> tmp;
+	for (auto ch : src)
+		encodeUTF16(ch, tmp);
 	return CFStringCreateWithCharacters(kCFAllocatorDefault, tmp.data(), tmp.size());
 }
-#endif
+#endif // #if LOG4CXX_CFSTRING_API
 
 
 logchar Transcoder::decode(char val)
@@ -688,13 +693,11 @@ std::string Transcoder::encodeCharsetName(const LogString& val)
 		};
 	std::string out;
 
-	for (LogString::const_iterator iter = val.begin();
-		iter != val.end();
-		iter++)
+	for (auto& item : val)
 	{
-		if (*iter >= 0x20 && *iter < 0x7F)
+		if (item >= 0x20 && item < 0x7F)
 		{
-			out.append(1, asciiTable[*iter - 0x20]);
+			out.append(1, asciiTable[item - 0x20]);
 		}
 		else
 		{
