@@ -26,7 +26,7 @@
 #include <thread>
 #include <condition_variable>
 
-namespace log4cxx
+namespace LOG4CXX_NS
 {
 namespace helpers
 {
@@ -42,7 +42,7 @@ class LOG4CXX_EXPORT FileWatchdog
 		/**
 		The default delay between every file modification check, set to 60
 		seconds.  */
-		static long DEFAULT_DELAY /*= 60000*/;
+		static long DEFAULT_DELAY /*= 60000 ms*/;
 
 	protected:
 		FileWatchdog(const File& filename);
@@ -52,12 +52,25 @@ class LOG4CXX_EXPORT FileWatchdog
 
 	public:
 		/**
-		Set the delay to observe between each check of the file changes.
+		Use \c delay as the number of milliseconds to wait between each check for file changes.
 		*/
-		void setDelay(long delay1);
+		void setDelay(long delay);
 
+		/**
+		Create a thread that periodically checks for a file change after first calling doOnChange() on the current thread.
+		*/
 		void start();
 
+		/**
+		Stop the thread that periodically checks for a file change.
+		*/
+		void stop();
+
+		/**
+		Is the thread that periodically checks for a file change running?
+		*/
+		bool is_active();
+
 	private:
 		void run();
 		bool is_interrupted();
